searchData={"content_type":"text/markdown","items":[{"doc":"Well-structured Arrays with fast random-element-access for Elixir, offering a common interface with multiple implementations with varying performance guarantees that can be switched in your configuration.","ref":"Arrays.html","title":"Arrays","type":"module"},{"doc":"#","ref":"Arrays.html#module-using-arrays","title":"Using `Arrays` - Arrays","type":"module"},{"doc":"##","ref":"Arrays.html#module-some-simple-examples","title":"Some simple examples: - Arrays","type":"module"},{"doc":"By calling `Arrays.new` or `Arrays.empty`:\n\n    iex> Arrays.new([\"Dvorak\", \"Tchaikovsky\", \"Bruch\"])\n    #Arrays.Implementations.MapArray \n\nBy using `Collectable`:\n\n    iex> [1, 2, 3] |> Enum.into(Arrays.new())\n    #Arrays.Implementations.MapArray \n    iex> for x  \n\n##","ref":"Arrays.html#module-constructing-arrays","title":"Constructing Arrays - Arrays","type":"module"},{"doc":"- Indexing is fast.\n- The full Access calls are supported,\n- Variants of many common `Enum`-like functions that keep the result an array (rather than turning it into a list), are available.\n\n      iex> words = Arrays.new([\"the\", \"quick\", \"brown\", \"fox\", \"jumps\", \"over\", \"the\", \"lazy\", \"dog\"])\n      #Arrays.Implementations.MapArray \n      iex> Arrays.size(words) # Runs in constant-time\n      9\n      iex> words[3] # Indexing is fast\n      \"fox\"\n      iex> words = put_in(words[2], \"purple\") # All of `Access` is supported\n      #Arrays.Implementations.MapArray \n      iex> Arrays.map(words, &String.upcase/1) # Map a function, keep result an array\n      #Arrays.Implementations.MapArray \n      iex> lengths = Arrays.map(words, &String.length/1)\n      #Arrays.Implementations.MapArray \n      iex> Arrays.reduce(lengths, 0, &Kernel.+/2) # `reduce_right` is supported as well.\n      36\n\nConcatenating arrays:\n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.concat(Arrays.new([4, 5, 6]))\n    #Arrays.Implementations.MapArray \n\nSlicing arrays:\n\n    iex> ints = Arrays.new(1..100)\n    iex> Arrays.slice(ints, 9..19)\n    #Arrays.Implementations.MapArray \n\n#","ref":"Arrays.html#module-some-common-array-operations","title":"Some common array operations: - Arrays","type":"module"},{"doc":"Algorithms that use arrays can be used while abstracting away from the underlying representation.\nWhich array implementation/representation is actually used, can then later be configured/compared, to make a trade-off between ease-of-use and time/memory efficiency.\n\n`Arrays` itself comes with two built-in implementations:\n\n- `Arrays.Implementations.ErlangArray` wraps the Erlang `:array` module, allowing this time-tested implementation to be used with all common Elixir protocols and syntactic sugar.\n- `Arrays.Implementations.MapArray` is a simple implementation that uses a map with sequential integers as keys.\n\nBy default, Elixir.Arrays.Implementations.MapArray is used when creating new array objects, but this can be configured by either changing the default in your whole application, or by passing an option to a specific invocation of [`new/0-2`](`new/0`), or [`empty/0-1`](`empty/0`).\n\n\nImplementations provided by other libraries:\n\n- [ArraysAja](https://github.com/Qqwy/elixir-arrays_aja) adds support for [Aja](https://github.com/sabiwara/aja)'s `A.Vector`, which is an implementation of a 'Hickey Trie' vector. For most operations, it significantly outperforms `ErlangArray` and `MapArray`.\n\n\n#","ref":"Arrays.html#module-rationale","title":"Rationale - Arrays","type":"module"},{"doc":"Besides being able to use all functions in this module,\none can use the following protocols and behaviours with them:\n\n- From Elixir's standard library:\n  - `Enumerable`: Iterating over arrays\n  - `Collectable`: Creating arrays from collections\n  - the `Access` behaviour: access a particular element using square brackets, `put_in` etc.\n\n- From common container libraries:\n  - `Insertable`: Append a single item from the end of an array\n  - `Extractable`: Take a single item from the end of an array\n  - `FunLand.Mappable`: Map a function over each element in the array, creating a new array with the results\n  - `FunLand.Reducible`: Reduce an array to a single value.\n\n\nNote: `FunLand` is an optional dependency of this library, so its functionality will only be available if `:fun_land` is also added to your `mix.exs` dependencies list.\n\n##","ref":"Arrays.html#module-protocols","title":"Protocols - Arrays","type":"module"},{"doc":"iex> myarray = Arrays.new([2, 1, 4, 2, 0])\n    iex> Enum.sort(myarray)\n    [0, 1, 2, 2, 4]\n    iex> Enum.count(myarray)\n    5\n    iex> Enum.with_index(myarray)\n    [{2, 0}, {1, 1}, {4, 2}, {2, 3}, {0, 4}]\n    iex> Enum.slice(myarray, 1, 3)\n    [1, 4, 2]\n\n    iex> names = Arrays.new([\"Ernie\", \"Bert\", \"Kermit\"])\n    iex> names |> Stream.map(&String.upcase/1) |> Enum.into(Arrays.new())\n    #Arrays.Implementations.MapArray \n\n    iex> foods = Arrays.new([\"Cheese\", \"Strawberries\", \"Cookies\"])\n    iex> foods |> Enum.take(2)\n    [\"Cheese\", \"Strawberries\"]\n\n    iex> Arrays.new([1, 2, 3]) |> Stream.zip(Arrays.new([4, 5, 6])) |> Enum.take(2)\n    [{1, 4}, {2, 5}]\n\n\n##","ref":"Arrays.html#module-enumerable","title":"Enumerable - Arrays","type":"module"},{"doc":"iex> [10, 20, 30, 40] |> Enum.into(Arrays.new())\n    #Arrays.Implementations.MapArray \n\n\n##","ref":"Arrays.html#module-collectable","title":"Collectable - Arrays","type":"module"},{"doc":"Fast random-element access and updates are supported.\n\n\n    iex> arr = Arrays.new([1, 2, 3, 4])\n    iex> arr = put_in(arr[2], 33)\n    #Arrays.Implementations.MapArray \n    iex> arr = update_in(arr[1], (&(&1 * -2)))\n    #Arrays.Implementations.MapArray \n    iex> update_in(arr[-1], (&(&1 + 1)))\n    #Arrays.Implementations.MapArray \n\nPopping from a random location however, is not.\nOnly removals of the last element of the array are fast.\nFor this, use `Arrays.extract/1`.\n\n    iex> arr = Arrays.new([1, -4, 33, 5])\n    iex> {33, _arr} = pop_in(arr[-2])\n    ** (ArgumentError) There is no efficient implementation possible to remove an element from a random location in an array, so `Access.pop/2` (and returning `:pop` from `Access.get_and_update/3` ) are not supported by Arrays.Implementations.MapArray. If you want to remove the last element, use `Arrays.extract/1`.\n\n    iex> arr2 = Arrays.new([10, 20, 30])\n    iex> {20, _arr2} = get_and_update_in(arr2[1], fn _ -> :pop end)\n    ** (ArgumentError) There is no efficient implementation possible to remove an element from a random location in an array, so `Access.pop/2` (and returning `:pop` from `Access.get_and_update/3` ) are not supported by Arrays.Implementations.MapArray. If you want to remove the last element, use `Arrays.extract/1`.\n\n    iex> arr2 = Arrays.new([10, 20, 30])\n    iex> {:ok, {value, arr2}} = Arrays.extract(arr2)\n    iex> value\n    30\n    iex> arr2\n    #Arrays.Implementations.MapArray \n\n\n\nsquare-bracket access, `get_in`, `put_in` and `update_in` are very fast operations.\nUnless `pop`/`pop_in` is used for the last element in the array, is a very slow operation,\nas it requires moving of all elements after the given index in the array.\n\nBoth positive indexes (counting from zero) and negative indexes\n(`-1` is the last element, `-2` the second-to-last element, etc.) are supported.\n\nHowever, if `positive_index > Arrays.size(array)` or `negative_index   arr = Arrays.new([1,2,3,4])\n    iex> put_in(arr[4], 1234)\n    ** (ArgumentError) argument error\n\n    iex> arr = Arrays.new([1,2,3,4])\n    iex> put_in(arr[-5], 100)\n    ** (ArgumentError) argument error\n\n    iex> arr = Arrays.new([1,2,3,4])\n    iex> Access.fetch(arr, 4)\n    :error\n    iex> Access.fetch(arr, -5)\n    :error\n    iex> arr[4]\n    nil\n    iex> arr[-5]\n    nil\n\n    iex> arr = Arrays.new([1,2,3,4])\n    iex> update_in(arr[8], fn x -> x * 2 end)\n    ** (ArgumentError) argument error\n\n    iex> arr = Arrays.new([1,2,3,4])\n    iex> update_in(arr[-8], fn x -> x * 2 end)\n    ** (ArgumentError) argument error\n\n##","ref":"Arrays.html#module-access","title":"Access - Arrays","type":"module"},{"doc":"iex> arr = Arrays.new()\n    iex> {:ok, arr} = Insertable.insert(arr, 42)\n    iex> {:ok, arr} = Insertable.insert(arr, 100)\n    iex> arr\n    #Arrays.Implementations.MapArray \n\n##","ref":"Arrays.html#module-insertable","title":"Insertable - Arrays","type":"module"},{"doc":"iex> Extractable.extract(Arrays.new())\n    {:error, :empty}\n    iex> {:ok, {3, arr}} = Extractable.extract(Arrays.new([1, 2, 3]))\n    iex> arr\n    #Arrays.Implementations.MapArray \n\n\n##","ref":"Arrays.html#module-extractable","title":"Extractable - Arrays","type":"module"},{"doc":"Note: `FunLand` is an optional dependency of this library.\n\n    iex> Arrays.new([1,2,3,4]) |> FunLand.reduce(0, &(&1+&2))\n    10\n\n##","ref":"Arrays.html#module-funland-reducible","title":"FunLand.Reducible - Arrays","type":"module"},{"doc":"iex> Arrays.new([1, 2, 3, 4]) |> FunLand.Mappable.map(fn x -> x * 2 end)\n    #Arrays.Implementations.MapArray","ref":"Arrays.html#module-funland-mappable","title":"FunLand.Mappable - Arrays","type":"module"},{"doc":"Elixir widely uses `List` as default collection type.\nCompared to lists, Arrays have the folowing differences:\n\n- Arrays keep track of their *size*. The size of a list needs to be computed.\n- Arrays allow fast¹ element *indexing*. Indexing later elements in a list slows down linearly in the size of the list.\n- Arrays allow fast *slicing*. For lists, this slows down the further away from the head of the list we are.\n- Pushing a single element to the _end_ of an array is fast¹. Pushing a single element to the end of a list is very slow (the whole list needs to be copied), taking linear time.\n- *Pushing* a single element to the _start_ of an array is slow, taking linear time (the whole array needs to be moved around). Pushing a single element to the head of a list is fast, taking constant time.\n- *Concatenating* arrays takes time proportional to the size of the second array (individual elements are pushed to the end). Concatenating two lists takes time proportional to the length of the first list. This means that repeated appending\n- Arrays are always well-formed. In certain cases, Lists are allowed to be improper.\n- Many common operations in Elixir transform an enumerable into a list automatically. Arrays are made using `Arrays.new/0`, `Arrays.new/1` `Arrays.empty/0`, the `into:` option on a `for`, or `Enum.into`.\n\n¹: Depending on the implementation, 'fast' is either _O(1)_ (constant time, regardless of array size) or _O(log(n))_ (logarithmic time, becoming a constant amount slower each time the array doubles in size.)\n\nThe linear time many operations on lists take, means that the operation becomes twice as slow when the list doubles in size.","ref":"Arrays.html#module-arrays-vs-lists","title":"Arrays vs Lists - Arrays","type":"module"},{"doc":"To add array-functionality to a custom datastructure, you'll need to implement the `Arrays.Protocol`.\n\nBesides these, you probably want to implement the above-mentioned protocols as well.\nYou can look at the source code of `Arrays.CommonProtocolImplementations` for some hints as to how those protocols can be easily implemented, as many functions can be defined as simple wrappers on top of the functions that `Arrays.Protocol` itself already provides.","ref":"Arrays.html#module-implementing-a-new-array-type","title":"Implementing a new Array type - Arrays","type":"module"},{"doc":"Appends ('pushes') a single element to the end of the array.\n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.append(4)\n    #Arrays.Implementations.MapArray \n\nSee also `extract/1`.","ref":"Arrays.html#append/2","title":"Arrays.append/2","type":"function"},{"doc":"Turns an array of arrays (or other enumerable of enumerables) into a single array.\n\n    iex> arr = Arrays.new([1, 2, 3])\n    iex> Arrays.concat([arr, arr, arr])\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.concat([])\n    #Arrays.Implementations.MapArray \n\nSee also `concat/2`.","ref":"Arrays.html#concat/1","title":"Arrays.concat/1","type":"function"},{"doc":"Returns an array where all elements of `right` are added to the end of `left`.\n\n`left` should be an array. `right` can be either an array or any other enumerable.\n\nTakes time proportional to the number of elements in `right`.\nEssentially, each element in `right` is appended to `left` in turn.\n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.concat(Arrays.new([4, 5, 6]))\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.concat((1..10))\n    #Arrays.Implementations.MapArray","ref":"Arrays.html#concat/2","title":"Arrays.concat/2","type":"function"},{"doc":"Creates a new, empty, array.\n\n    iex> Arrays.empty()\n    #Arrays.Implementations.MapArray \n\n\n#","ref":"Arrays.html#empty/1","title":"Arrays.empty/1","type":"function"},{"doc":"- `implementation:` Module name of array-implementation to use.\n- When not specified, will use the implementation which is configured in `:arrays, :default_array_implementation`,\n- When no configuration is specified either, Elixir.Arrays.Implementations.MapArray will be used by default.\n\n      iex> Arrays.empty([implementation: Arrays.Implementations.MapArray])\n      #Arrays.Implementations.MapArray \n\n      iex> Arrays.empty([implementation: Arrays.Implementations.ErlangArray])\n      #Arrays.Implementations.ErlangArray \n\nAny other option is passed on to the particular array implementation.\nNot all array implementations recognize all options.\nHowever, the following two options are very common (and supported by both built-in implementations, `Arrays.Implementations.ErlangArray` and `Arrays.Implementations.MapArray`):\n\n- `default:` Value that empty elements should start with. (Default: `nil`)\n- `size:` Size of the array at start. (Default: 0)\n\n##","ref":"Arrays.html#empty/1-options","title":"Options - Arrays.empty/1","type":"function"},{"doc":"iex> Arrays.empty([default: 42, size: 5, implementation: Arrays.Implementations.MapArray])\n    #Arrays.Implementations.MapArray \n\n##","ref":"Arrays.html#empty/1-using-the-maparray","title":"Using the MapArray - Arrays.empty/1","type":"function"},{"doc":"iex> Arrays.empty([default: \"Empty\" , size: 1, implementation: Arrays.Implementations.ErlangArray])\n    #Arrays.Implementations.ErlangArray","ref":"Arrays.html#empty/1-using-the-erlangarray","title":"Using the ErlangArray - Arrays.empty/1","type":"function"},{"doc":"Extracts ('pops') a single element from the end of the array.\n\nReturns `{:ok, item_that_was_removed, array_without_item}` if the array was non-empty.\nWhen called on an empty array, `{:error, :empty}` is returned.\n\n    iex> {:ok, {elem, arr}} = Arrays.new([1,2,3,4]) |> Arrays.extract()\n    iex> elem\n    4\n    iex> arr\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.new([]) |> Arrays.extract()\n    {:error, :empty}\n\nSee also `append/2`.","ref":"Arrays.html#extract/1","title":"Arrays.extract/1","type":"function"},{"doc":"Retrieves the value stored in `array` of the element at `index`.\n\nArray indexes start at *zero*.\n\n    iex> Arrays.new([3, 6, 9]) |> Arrays.get(0)\n    3\n\n    iex> Arrays.new([3, 6, 9]) |> Arrays.get(1)\n    6\n\nAs Array types also implement the `Access` behaviour,\nthe `[]` (square-bracket) syntactic sugar can also be used:\n\n    iex> myarray = Arrays.new([2, 4, 8])\n    iex> myarray[0]\n    2\n    iex> myarray[1]\n    4","ref":"Arrays.html#get/2","title":"Arrays.get/2","type":"function"},{"doc":"It is also possible to use negative indexes, to read elements starting from the right side of the array.\nFor example, index `-1` works equivalently to `size - 1`, if your array has `size`  elements.\n\n\n    iex> names = Arrays.new([\"Alice\", \"Bob\", \"Charlie\", \"David\"])\n    iex> Arrays.get(names, -2)\n    \"Charlie\"\n    iex> names[-1]\n    \"David\"","ref":"Arrays.html#get/2-negative-indexes","title":"Negative indexes - Arrays.get/2","type":"function"},{"doc":"Maps a function over an array, returning a new array.\n\n    iex> Arrays.new([1,2,3]) |> Arrays.map(fn val -> val * 2 end)\n    #Arrays.Implementations.MapArray","ref":"Arrays.html#map/2","title":"Arrays.map/2","type":"function"},{"doc":"Creates a new, empty array with default options.\n\n    iex> Arrays.new()\n    #Arrays.Implementations.MapArray","ref":"Arrays.html#new/0","title":"Arrays.new/0","type":"function"},{"doc":"Creates a new array, receiving its elements from the given `Enumerable`, with default options.","ref":"Arrays.html#new/1","title":"Arrays.new/1","type":"function"},{"doc":"Creates a new array, receiving its elements from the given `Enumerable`, with the given options.\n\nWhich options are supported depends on the type of array.\n\n    iex> Arrays.new([1, 2, 3])\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.new([\"Hello\"])\n    #Arrays.Implementations.MapArray","ref":"Arrays.html#new/2","title":"Arrays.new/2","type":"function"},{"doc":"Reduce an array to a single value, by calling the provided accumulation function for each element, left-to-right.\n\nIf the array is empty, the accumulator argument `acc` is returned as result immediately.\nOtherwise, the function is called on all elements in the array, in order, with `acc` as *second* argument.\nThe result of each of these function calls creates the new accumulator passed to the next invocation.\n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.reduce(0, fn val, sum -> sum + val end)\n    6\n\n    iex> Arrays.new([\"the\", \"quick\", \"brown\", \"fox\"]) |> Arrays.reduce(\"\", fn val, result -> result   \" \"   val end)\n    \" the quick brown fox\"\n\n    iex> Arrays.new([]) |> Arrays.reduce(1234, fn val, mul -> mul * val end)\n    1234\n\nSee also `reduce_right/3`.","ref":"Arrays.html#reduce/3","title":"Arrays.reduce/3","type":"function"},{"doc":"Reduce an array to a single value, by calling the provided accumulation function for each element, right-to-left.\n\n\nIf the array is empty, the accumulator argument `acc` is returned as result immediately.\nOtherwise, the function is called on all elements in the array, in reverse (right-to-left) order, with `acc` as *first* argument.\nThe result of each of these function calls creates the new accumulator passed to the next invocation.\n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.reduce_right(0, fn sum, val -> sum + val end)\n    6\n\n    iex> Arrays.new([\"the\", \"quick\", \"brown\", \"fox\"]) |> Arrays.reduce_right(\"\", fn result, val -> result   \" \"   val end)\n    \" fox brown quick the\"\n\n    iex> Arrays.new([]) |> Arrays.reduce_right(1234, fn mul, val -> mul * val end)\n    1234\n\n\nSee also `reduce/3`.","ref":"Arrays.html#reduce_right/3","title":"Arrays.reduce_right/3","type":"function"},{"doc":"Replaces the element in `array` at `index` with `value`.\n\n\n    iex> Arrays.new([4, 5, 6]) |> Arrays.replace(1, 69)\n    #Arrays.Implementations.MapArray \n\nJust like `get/2`, negative indices are supported.\n\n    iex> Arrays.new([7, 8, 9]) |> Arrays.replace(-1, 33)\n    #Arrays.Implementations.MapArray","ref":"Arrays.html#replace/3","title":"Arrays.replace/3","type":"function"},{"doc":"Changes the size of the array.\n\nWhen the array becomes larger, new elements at the end will al receive the `default` value.\nWhen the array becomes smaller, elements larger than the new `size` will be dropped.\n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.resize(6)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.resize(5, 42)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.resize(1)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.new([9, 8, 7]) |> Arrays.resize(0)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.new([1, 2, 3]) |> Arrays.resize(3)\n    #Arrays.Implementations.MapArray \n\nSee also `size/1`.","ref":"Arrays.html#resize/3","title":"Arrays.resize/3","type":"function"},{"doc":"The number of elements in the array.\n\nLooking up the size of an array is fast: this function runs in constant time.\n\n    iex> Arrays.new([2, 4, 6]) |> Arrays.size()\n    3\n\n    iex> Arrays.new([]) |> Arrays.size()\n    0\n\nSee also `resize/2`.","ref":"Arrays.html#size/1","title":"Arrays.size/1","type":"function"},{"doc":"Returns a sub-array of the given array by `index_range`.\n\n`index_range` must be a `Range`. Given an array, it drops elements before\n`index_range.first` (zero-based), then it takes elements until element\n`index_range.last` (inclusively).\n\nIndexes are normalized, meaning that negative indexes will be counted from the\nend (for example, -1 means the last element of the array).\n\nIf `index_range.last` is out of bounds, then it is assigned as the index of the\nlast element.\n\nIf the normalized `index_range.first` is out of bounds of the given array,\nor if it is is greater than the normalized `index_range.last`,\nthen an empty array is returned.\n\n    iex> Arrays.slice(Arrays.new([1, 2, 3, 4, 5, 6]), 2..4)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.slice(Arrays.new(1..100), 5..10)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.slice(Arrays.new(1..10), 5..20)\n    #Arrays.Implementations.MapArray \n\n    # last five elements (negative indexes)\n    iex> Arrays.slice(Arrays.new(1..30), -5..-1)\n    #Arrays.Implementations.MapArray \n\nIf values are out of bounds, it returns an empty array:\n\n    iex> Arrays.slice(Arrays.new(1..10), 11..20)\n    #Arrays.Implementations.MapArray \n\n    # first is greater than last\n    iex> Arrays.slice(Arrays.new(1..10), 6..5)\n    #Arrays.Implementations.MapArray \n\nSee also `slice/3`.\n\nThis function is similar to `Enum.slice/2`, but will always return an array (whereas `Enum.slice` will return a list).\n\nSlicing on arrays is significantly faster than slicing on plain lists or maps, as there is no need to iterate over all elements we're going to skip since we can index them directly.","ref":"Arrays.html#slice/2","title":"Arrays.slice/2","type":"function"},{"doc":"Returns a sub-array of the given array, from `start_index` (zero-based)\nwith `amount` number of elements if available.\n\nGiven an array, it skips elements right before element `start_index`; then,\nit takes `amount` of elements, returning as many elements as possible if there\nare not enough elements.\n\nA negative `start_index` can be passed, which means the array is enumerated\nonce and the index is counted from the end (for example, -1 starts slicing from\nthe last element).\n\nIt returns an empty array if `amount` is 0 or if `start_index` is out of bounds.\n\n    iex> Arrays.slice(Arrays.new([1, 2, 3, 4, 5, 6]), 2, 4)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.slice(Arrays.new([1, 2, 3, 4, 5, 6]), 10, 20)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.slice(Arrays.new(1..100), 5, 10)\n    #Arrays.Implementations.MapArray \n\n    # amount to take is greater than the number of elements\n    iex> Arrays.slice(Arrays.new(1..10), 5, 100)\n    #Arrays.Implementations.MapArray \n\n    iex> Arrays.slice(Arrays.new(1..10), 5, 0)\n    #Arrays.Implementations.MapArray \n\n    # using a negative start index\n    iex> Arrays.slice(Arrays.new(1..10), -6, 3)\n    #Arrays.Implementations.MapArray \n\n    # out of bound start index (positive)\n    iex> Arrays.slice(Arrays.new(1..10), 10, 5)\n    #Arrays.Implementations.MapArray \n\n    # out of bound start index (negative)\n    iex> Arrays.slice(Arrays.new(1..10), -11, 5)\n    #Arrays.Implementations.MapArray \n\nSee also `slice/2`.\n\nThis function is similar to `Enum.slice/3`, but will always return an array (whereas `Enum.slice` will return a list).\n\nSlicing on arrays is significantly faster than slicing on plain lists or maps, as there is no need to iterate over all elements we're going to skip since we can index them directly.","ref":"Arrays.html#slice/3","title":"Arrays.slice/3","type":"function"},{"doc":"Transforms the array into a list.\n\n    iex> Arrays.new([\"Joe\", \"Mike\", \"Robert\"]) |> Arrays.to_list\n    [\"Joe\", \"Mike\", \"Robert\"]","ref":"Arrays.html#to_list/1","title":"Arrays.to_list/1","type":"function"},{"doc":"Any datatype implementing the `Arrays.Protocol`.","ref":"Arrays.html#t:array/0","title":"Arrays.array/0","type":"type"},{"doc":"An array of elements of type `element`.\n\nThis type is equivalent to `t:array/0` but is especially useful for documentation.\n\nFor example, imagine you define a function that expects an array of\nintegers and returns an array of strings:\n\n    @spec integers_to_strings(Arrays.array(integer())) :: Arrays.array(String.t())\n    def integers_to_strings(integers) do\n      Arrays.map(integers, &Integer.to_string/1)\n    end","ref":"Arrays.html#t:array/1","title":"Arrays.array/1","type":"type"},{"doc":"An array index can be either a nonnegative index (up to the size of the array),\nor a negative index (then we count backwards from the size.)","ref":"Arrays.html#t:index/0","title":"Arrays.index/0","type":"type"},{"doc":"Type of the kind of value stored in the array.\nIn practice, arrays can store anything so this is an alias for `any`.","ref":"Arrays.html#t:value/0","title":"Arrays.value/0","type":"type"},{"doc":"Wraps the Erlang [`:array`](http://erlang.org/doc/man/array.html) module.\n\nThese kinds of arrays use a 'functional tree' format,\nwith a leaf size of 10 nodes wide ([ref.](https://github.com/erlang/otp/blob/maint/lib/stdlib/src/array.erl#L108)).\n\nCommon operations like element access thus take O(log10(n)) time.\n\nNote that when no custom default value is specified,\n`nil` will be used, rather than `:array`'s default of `:undefined`.","ref":"Arrays.Implementations.ErlangArray.html","title":"Arrays.Implementations.ErlangArray","type":"module"},{"doc":"Create an `%ErlangArray{}`-struct from an `:array`-record.\n\n\n    iex> :array.new([]) |> ErlangArray.from_raw()\n    #Arrays.Implementations.ErlangArray","ref":"Arrays.Implementations.ErlangArray.html#from_raw/1","title":"Arrays.Implementations.ErlangArray.from_raw/1","type":"function"},{"doc":"Implementation for `FunLand.Mappable.map`.\n\nNote that `FunLand` is an optional dependency of `Arrays` so you need to add it to your `mix.exs` dependencies manually to use it.","ref":"Arrays.Implementations.ErlangArray.html#map/2","title":"Arrays.Implementations.ErlangArray.map/2","type":"function"},{"doc":"A variant of reduce that accepts anything that is Combinable\nas second argument. This Combinable will determine what the empty value and the\ncombining operation will be.\n\nPass in the combinable module name to start with `empty` as accumulator,\nor the combinable as struct to use that as starting accumulator.\n\nThis is an automatic function implementation, made possible because Arrays.Implementations.ErlangArray\nimplements the `FunLand.Reducible` behaviour.\n\nSee `FunLand.Reducible.reduce/2` for examples.","ref":"Arrays.Implementations.ErlangArray.html#reduce/2","title":"Arrays.Implementations.ErlangArray.reduce/2","type":"function"},{"doc":"Implementation for `FunLand.Reducible.reduce`.\n\nNote that `FunLand` is an optional dependency of `Arrays` so you need to add it to your `mix.exs` dependencies manually to use it.","ref":"Arrays.Implementations.ErlangArray.html#reduce/3","title":"Arrays.Implementations.ErlangArray.reduce/3","type":"function"},{"doc":"Converts the reducible into a list,\nby building up a list from all elements, and in the end reversing it.\n\nThis is an automatic function implementation, made possible because Arrays.Implementations.ErlangArray\nimplements the `FunLand.Reducible` behaviour.","ref":"Arrays.Implementations.ErlangArray.html#to_list/1","title":"Arrays.Implementations.ErlangArray.to_list/1","type":"function"},{"doc":"Turn an %ErlangArray{}-struct back into an `:array`-record.\n\n      iex> Arrays.new([1, 2, 3], implementation: Arrays.Implementations.ErlangArray) |> ErlangArray.to_raw()\n      {:array, 3, 10, nil, {1, 2, 3, nil, nil, nil, nil, nil, nil, nil}}","ref":"Arrays.Implementations.ErlangArray.html#to_raw/1","title":"Arrays.Implementations.ErlangArray.to_raw/1","type":"function"},{"doc":"An array implementation based on the built-in Map structure.","ref":"Arrays.Implementations.MapArray.html","title":"Arrays.Implementations.MapArray","type":"module"},{"doc":"Implementation for `FunLand.Mappable.map`.\n\nNote that `FunLand` is an optional dependency of `Arrays` so you need to add it to your `mix.exs` dependencies manually to use it.","ref":"Arrays.Implementations.MapArray.html#map/2","title":"Arrays.Implementations.MapArray.map/2","type":"function"},{"doc":"A variant of reduce that accepts anything that is Combinable\nas second argument. This Combinable will determine what the empty value and the\ncombining operation will be.\n\nPass in the combinable module name to start with `empty` as accumulator,\nor the combinable as struct to use that as starting accumulator.\n\nThis is an automatic function implementation, made possible because Arrays.Implementations.MapArray\nimplements the `FunLand.Reducible` behaviour.\n\nSee `FunLand.Reducible.reduce/2` for examples.","ref":"Arrays.Implementations.MapArray.html#reduce/2","title":"Arrays.Implementations.MapArray.reduce/2","type":"function"},{"doc":"Implementation for `FunLand.Reducible.reduce`.\n\nNote that `FunLand` is an optional dependency of `Arrays` so you need to add it to your `mix.exs` dependencies manually to use it.","ref":"Arrays.Implementations.MapArray.html#reduce/3","title":"Arrays.Implementations.MapArray.reduce/3","type":"function"},{"doc":"Converts the reducible into a list,\nby building up a list from all elements, and in the end reversing it.\n\nThis is an automatic function implementation, made possible because Arrays.Implementations.MapArray\nimplements the `FunLand.Reducible` behaviour.","ref":"Arrays.Implementations.MapArray.html#to_list/1","title":"Arrays.Implementations.MapArray.to_list/1","type":"function"},{"doc":"This module is only interesting for people wanting to implement their own array datatype.\n\nIt does not contain any functions itself directly, but its source code might be useful to see example implementations\nof the various protocols and behaviours that Array types ought to implement.\nMost of these can probably be re-used 1:1 for custom Array types.","ref":"Arrays.CommonProtocolImplementations.html","title":"Arrays.CommonProtocolImplementations","type":"module"},{"doc":"This protocol is implemented by all array types.\n\nDo not call functions in this module directly if you want to use an array in your code.\nInstead, use the functions in the `Arrays` module, which will use the methods of this protocol.","ref":"Arrays.Protocol.html","title":"Arrays.Protocol","type":"protocol"},{"doc":"Appends ('pushes') a single element to the end of the array.\n\nCalled by `Arrays.append/2`","ref":"Arrays.Protocol.html#append/2","title":"Arrays.Protocol.append/2","type":"function"},{"doc":"Should create a new instance of your custom array type.\n\nThis is called internally by functions such as `Arrays.new/0` and `Arrays.empty/1`.\n\nNOTE: This function will not be dispatched by normal protocol handling.\nIt will be called directly:\nThe first (and only) parameter will be a list of options.\n\nc.f. `t:options`.","ref":"Arrays.Protocol.html#empty/1","title":"Arrays.Protocol.empty/1","type":"function"},{"doc":"Extracts ('pops') a single element from the end of the array.\n\nCalled by `Arrays.extract/1`","ref":"Arrays.Protocol.html#extract/1","title":"Arrays.Protocol.extract/1","type":"function"},{"doc":"Retrieves the value stored in `array` of the element at `index`.\n\nCalled by `Arrays.get/2`","ref":"Arrays.Protocol.html#get/2","title":"Arrays.Protocol.get/2","type":"function"},{"doc":"Maps a function over an array, returning a new array.\n\nCalled by `Arrays.map/2`","ref":"Arrays.Protocol.html#map/2","title":"Arrays.Protocol.map/2","type":"function"},{"doc":"Reduce an array to a single value, by calling the provided accumulation function for each element, left-to-right.\n\nNote that `fun` takes the accumulator as _second_ (right) parameter and the item as _first_ (left) parameter.\n\nCalled by `Arrays.reduce/3`","ref":"Arrays.Protocol.html#reduce/3","title":"Arrays.Protocol.reduce/3","type":"function"},{"doc":"Reduce an array to a single value, by calling the provided accumulation function for each element, right-to-left.\n\nNote that `fun` takes the accumulator as _first_ (left) parameter and the item as _second_ (right) parameter.\n\nCalled by `Arrays.reduce_right/3`","ref":"Arrays.Protocol.html#reduce_right/3","title":"Arrays.Protocol.reduce_right/3","type":"function"},{"doc":"Replaces the element in `array` at `index` with `value`.\n\nCalled by `Arrays.replace/3`","ref":"Arrays.Protocol.html#replace/3","title":"Arrays.Protocol.replace/3","type":"function"},{"doc":"Changes the size of the array.\n\nWhen made smaller, truncates elements beyond the first `size` elements will be removed.\nWhen made larger, new elements will receive `default` as value.\n\nCalled by `Arrays.resize/2`","ref":"Arrays.Protocol.html#resize/3","title":"Arrays.Protocol.resize/3","type":"function"},{"doc":"The number of elements in the array.\n\nCalled by `Arrays.size/1`","ref":"Arrays.Protocol.html#size/1","title":"Arrays.Protocol.size/1","type":"function"},{"doc":"Return a contiguous slice of some elements in the array.\n\nHandling of bounds is handled in the `Arrays` module,\nso we know for certain that `0 <= start_index < size(array)`\nand `start_index + length < size(array)`.","ref":"Arrays.Protocol.html#slice/3","title":"Arrays.Protocol.slice/3","type":"function"},{"doc":"Transforms the array into a list.\n\nCalled by `Arrays.to_list/1`","ref":"Arrays.Protocol.html#to_list/1","title":"Arrays.Protocol.to_list/1","type":"function"},{"doc":"Any datatype implementing the `Arrays.Protocol`.","ref":"Arrays.Protocol.html#t:array/0","title":"Arrays.Protocol.array/0","type":"type"},{"doc":"An array index can be either a nonnegative index (up to the size of the array),\nor a negative index (then we count backwards from the size.)","ref":"Arrays.Protocol.html#t:index/0","title":"Arrays.Protocol.index/0","type":"type"},{"doc":"A list of options passed to `c:empty/1`.\n\nWhat options are recognized by a particular implementation varies.","ref":"Arrays.Protocol.html#t:options/0","title":"Arrays.Protocol.options/0","type":"type"},{"doc":"","ref":"Arrays.Protocol.html#t:t/0","title":"Arrays.Protocol.t/0","type":"type"},{"doc":"Type of the kind of value stored in the array.\nIn practice, arrays can store anything so this is an alias for `any`.","ref":"Arrays.Protocol.html#t:value/0","title":"Arrays.Protocol.value/0","type":"type"}]}